/*
  Copyright (C) 2001-2003 Renaud Pawlak <renaud@aopsys.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package org.objectweb.jac.aspects.confirmation;

import org.objectweb.jac.aspects.gui.DisplayContext;
import org.objectweb.jac.aspects.gui.GuiAC;
import org.objectweb.jac.core.AspectComponent;
import org.objectweb.jac.core.Display;
import org.objectweb.jac.core.Interaction;
import org.objectweb.jac.core.Wrapper;
import org.objectweb.jac.core.rtti.AbstractMethodItem;
import org.objectweb.jac.core.rtti.NamingConventions;
import org.objectweb.jac.util.Log;

/**
 * The confirmation aspect implementation (allows the user to add
 * confirmation popups before committing. */

public class ConfirmationAC extends AspectComponent implements ConfirmationConf {

   static String cancellationMessage = "invocation was cancelled by the user";
   public void confirm(String classes, String methods, String objects) {
      pointcut(objects,classes,methods,
               new ConfirmationWrapper(this),"confirm","catchCancellation");
   }

   /**
    * A confirmation wrapper that wraps methods to show a confirmation
    * message box before actually performing the call. */
   
   public class ConfirmationWrapper extends Wrapper {

      public ConfirmationWrapper(AspectComponent ac) {
         super(ac);
      }

      /** The wrapping method. */
      public Object confirm(Interaction interaction) 
         throws OperationCanceledException
      {
         Log.trace("confirmation","confirm "+interaction);
         DisplayContext context = (DisplayContext)this.attr(GuiAC.DISPLAY_CONTEXT);
         Log.trace("confirmation","context="+context);
         AbstractMethodItem method = interaction.method;
         if (context!=null) {
            Display display = context.getDisplay();
            String message;
            if (method.isRemover()) {
               message = "Do you really want to remove "+
                  NamingConventions.textForName(method.getRemovedCollections()[0].getName())+" '"+
                  GuiAC.toString(interaction.args[0])+
                  "' from "+NamingConventions.textForName(interaction.getClassItem().getShortName())+" '"+
                  GuiAC.toString(interaction.wrappee)+"' ?";
            } else {
               message = "Do you really want to "+
                  NamingConventions.textForName(interaction.method.getName())+
                  (interaction.method.isStatic() ? "" : " for '"+
                   GuiAC.toString(interaction.wrappee)) +"' ?";
            }

            if (!display.showMessage(message,"Confirmation",true,true,false)) {
               throw new OperationCanceledException(interaction);
            }
         }
         return interaction.proceed();
      }
      
      /** The exception handler. */
      public void catchCancellation(OperationCanceledException e) {
         System.out.println("Catching exception: "+e);
      } 

   }

}

class OperationCanceledException extends Exception {
   public OperationCanceledException(Interaction interaction) {
      super("Operation cancelled: "+interaction.method);
   }
}
