/*
  Copyright (C) 2001-2003 Lionel Seinturier <Lionel.Seinturier@lip6.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package org.objectweb.jac.aspects.distrans;

import javax.transaction.NotSupportedException;
import javax.transaction.SystemException;
import javax.transaction.UserTransaction;

import org.objectweb.jac.core.AspectComponent;
import org.objectweb.jac.core.Interaction;
import org.objectweb.jac.core.Wrapper;

/**
 * This wrapper delimits the begining of a transaction.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lip6.fr>
 * @version 1.0
 */
public class BeginTransactionWrapper extends Wrapper {

    /** The transaction. */
    private UserTransaction usertx;
    
    /**
     * @param ac  the AC managing this wrapper
     */
    public BeginTransactionWrapper( AspectComponent ac ) {
        super(ac);
        usertx = JOTMHelper.get().getUserTransaction();
    }
    
    public Object begin( Interaction interaction ) {
        try {
			return _begin(interaction);
		} catch (NotSupportedException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (SystemException e) {
			e.printStackTrace();
			System.exit(1);
		}
        return null;
    }
    
    private Object _begin( Interaction interaction )
        throws NotSupportedException, SystemException {

		usertx.begin();

        return proceed(interaction);
    }
    
}
