/*
  Copyright (C) 2002-2003 Laurent Martelli <laurent@aopsys.com>
                          Renaud Pawlak <renaud@aopsys.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package org.objectweb.jac.aspects.gui;

import java.util.Arrays;
import java.util.Hashtable;
import org.objectweb.jac.core.NameRepository;
import org.objectweb.jac.core.rtti.*;
import org.objectweb.jac.util.ExtBoolean;
import org.objectweb.jac.util.Log;

/**
 * This is a generic view factory. It can be configured to use any
 * constructor to build views of any type.
 */
public class ViewFactory {

   /**
    * Init a swing factory by setting the constuctor types. */ 

   public static void init(String type,ViewFactory f) {
      f.setViewConstructor("Object", "org.objectweb.jac.aspects.gui.GenericFactory.createObjectView");
      f.setViewConstructor("NoTab", "org.objectweb.jac.aspects.gui.GenericFactory.createObjectViewNoTab");
      f.setViewConstructor ("InputParameters", "org.objectweb.jac.aspects.gui.GenericFactory.createParameters");
      //      f.setViewConstructor("java.lang.Throwable", "org.objectweb.jac.aspects.gui.ExceptionViewer.createExceptionViewer");
      if (type.equals("swing")) {
         f.setViewConstructor("ObjectView","org.objectweb.jac.aspects.gui.swing.ObjectView");
         f.setViewConstructor("Customized", "org.objectweb.jac.aspects.gui.swing.SwingCustomized");
         f.setViewConstructor("Reference", "org.objectweb.jac.aspects.gui.swing.ReferenceView(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("ObjectChooser", "org.objectweb.jac.aspects.gui.swing.ObjectChooser");
         f.setViewConstructor("Tree", "org.objectweb.jac.aspects.gui.swing.Tree");
         f.setViewConstructor("Panel", "org.objectweb.jac.aspects.gui.swing.SwingPanelView");
         f.setViewConstructor("Label", "org.objectweb.jac.aspects.gui.swing.SwingLabel");
         f.setViewConstructor("Tabbed", "org.objectweb.jac.aspects.gui.swing.SwingTabbedView");
         f.setViewConstructor("ParameterContainer", "org.objectweb.jac.aspects.gui.swing.SwingEditorContainer");
         f.setViewConstructor("Container", "org.objectweb.jac.aspects.gui.swing.SwingContainerView");
         f.setViewConstructor("SingleSlotContainer", "org.objectweb.jac.aspects.gui.swing.SingleSlotContainer");
         f.setViewConstructor("Desktop", "org.objectweb.jac.aspects.gui.swing.DesktopView");
         f.setViewConstructor("Method", "org.objectweb.jac.aspects.gui.swing.SwingMethodView");
         f.setViewConstructor("Field", "org.objectweb.jac.aspects.gui.swing.SwingFieldView");
         f.setViewConstructor("Enum", "org.objectweb.jac.aspects.gui.swing.EnumViewer(java.lang.Object,org.objectweb.jac.util.Enum,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("Table", "org.objectweb.jac.aspects.gui.swing.SwingTableView");
         f.setViewConstructor("List", "org.objectweb.jac.aspects.gui.swing.List");
         f.setViewConstructor("Dialog", "org.objectweb.jac.aspects.gui.swing.Dialog");
         f.setViewConstructor("MenuBar", "org.objectweb.jac.aspects.gui.swing.MenuBar()");
         f.setViewConstructor("Menu", "org.objectweb.jac.aspects.gui.swing.Menu()");
         f.setViewConstructor("ToolBar", "org.objectweb.jac.aspects.gui.swing.ToolBar");
         f.setViewConstructor("StatusBar", "org.objectweb.jac.aspects.gui.swing.StatusBar");

         f.setViewConstructor("Empty", "org.objectweb.jac.aspects.gui.swing.SwingEmptyView");
  
         f.setViewConstructor("dateHour", "org.objectweb.jac.aspects.gui.swing.DateHourViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("java.util.Date", "org.objectweb.jac.aspects.gui.swing.DateViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("imageURL", "org.objectweb.jac.aspects.gui.swing.ImageURLViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("text", "org.objectweb.jac.aspects.gui.swing.TextViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("java.lang.Float","org.objectweb.jac.aspects.gui.swing.FloatViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("java.lang.Double","org.objectweb.jac.aspects.gui.swing.FloatViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("double","org.objectweb.jac.aspects.gui.swing.FloatViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("float","org.objectweb.jac.aspects.gui.swing.FloatViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         
         f.setViewConstructor("org.objectweb.jac.lib.Attachment", "org.objectweb.jac.aspects.gui.swing.AttachmentViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");

         //editors
         f.setViewConstructor("editor:java.lang.Boolean", "org.objectweb.jac.aspects.gui.swing.BooleanEditor");
         f.setViewConstructor("editor:boolean", "org.objectweb.jac.aspects.gui.swing.BooleanEditor");
         f.setViewConstructor("editor:java.io.File", "org.objectweb.jac.aspects.gui.swing.FileEditor");
         f.setViewConstructor("editor:directory", "org.objectweb.jac.aspects.gui.swing.DirectoryEditor");
         f.setViewConstructor("editor:java.net.URL", "org.objectweb.jac.aspects.gui.swing.URLEditor");
         f.setViewConstructor("editor:java.util.Date", "org.objectweb.jac.aspects.gui.swing.DateEditor");
         f.setViewConstructor("editor:dateHour", "org.objectweb.jac.aspects.gui.swing.DateHourEditor");
         f.setViewConstructor("editor:text", "org.objectweb.jac.aspects.gui.swing.TextEditor");
         f.setViewConstructor("editor:javaCode", "org.objectweb.jac.aspects.gui.swing.JavaCodeEditor");
         f.setViewConstructor("editor:password", "org.objectweb.jac.aspects.gui.swing.PasswordFieldEditor");
         f.setViewConstructor("editor:org.objectweb.jac.lib.Attachment", "org.objectweb.jac.aspects.gui.swing.AttachmentEditor");
         f.setViewConstructor("PrimitiveTypeEditor", "org.objectweb.jac.aspects.gui.swing.PrimitiveFieldEditor");
         
         f.setViewConstructor("CollectionItemView", "org.objectweb.jac.aspects.gui.swing.CollectionItemView");
         
         // table cells viewers
         f.setViewConstructor("cell:java.util.Date", "org.objectweb.jac.aspects.gui.swing.DateViewer()");
         f.setViewConstructor("cell:dateHour", "org.objectweb.jac.aspects.gui.swing.DateHourViewer()");
         f.setViewConstructor("cell:imageURL", "org.objectweb.jac.aspects.gui.swing.ImageURLViewer()");
         f.setViewConstructor("cell:Reference", "org.objectweb.jac.aspects.gui.swing.ReferenceView()");
         f.setViewConstructor("cell:Enum", "org.objectweb.jac.aspects.gui.swing.EnumViewer(org.objectweb.jac.util.Enum)");
         f.setViewConstructor("cell:java.lang.Float", "org.objectweb.jac.aspects.gui.swing.FloatViewer()");
         f.setViewConstructor("cell:java.lang.Double", "org.objectweb.jac.aspects.gui.swing.FloatViewer()");
         f.setViewConstructor("cell:float", "org.objectweb.jac.aspects.gui.swing.FloatViewer()");
         f.setViewConstructor("cell:double", "org.objectweb.jac.aspects.gui.swing.FloatViewer()");
         f.setViewConstructor("cell:org.objectweb.jac.lib.Attachment", "org.objectweb.jac.aspects.gui.swing.AttachmentViewer()");
         f.setViewConstructor("cell:Method", "org.objectweb.jac.aspects.gui.swing.SwingMethodView");
         f.setViewConstructor("cell:List", "org.objectweb.jac.aspects.gui.swing.SwingEmptyView()");

      } else if(type.equals("web")) {
         
         f.setViewConstructor("ObjectView", "org.objectweb.jac.aspects.gui.web.ObjectView");
         f.setViewConstructor("Empty", "org.objectweb.jac.aspects.gui.web.Empty");
         f.setViewConstructor("Customized", "org.objectweb.jac.aspects.gui.web.Customized");
         f.setViewConstructor("Panel", "org.objectweb.jac.aspects.gui.web.Panel");
         f.setViewConstructor("Container", "org.objectweb.jac.aspects.gui.web.Container");
         f.setViewConstructor("SingleSlotContainer", "org.objectweb.jac.aspects.gui.web.SingleSlotContainer");
         f.setViewConstructor("Label", "org.objectweb.jac.aspects.gui.web.Label");
         f.setViewConstructor("Reference", "org.objectweb.jac.aspects.gui.web.ReferenceView(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("Enum", "org.objectweb.jac.aspects.gui.web.EnumViewer(java.lang.Object,org.objectweb.jac.util.Enum,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("Field", "org.objectweb.jac.aspects.gui.web.PrimitiveField");
         f.setViewConstructor("Table", "org.objectweb.jac.aspects.gui.web.Table");
         f.setViewConstructor("List", "org.objectweb.jac.aspects.gui.web.List");
         f.setViewConstructor("Tree", "org.objectweb.jac.aspects.gui.web.Tree");
         f.setViewConstructor("Tabbed", "org.objectweb.jac.aspects.gui.web.Tabs");
         f.setViewConstructor("Method", "org.objectweb.jac.aspects.gui.web.Method");
         f.setViewConstructor("EmbeddedMethod", "org.objectweb.jac.aspects.gui.web.EmbeddedMethod");
         f.setViewConstructor("Window", "org.objectweb.jac.aspects.gui.web.Page");
         f.setViewConstructor("Dialog", "org.objectweb.jac.aspects.gui.web.Dialog");
         f.setViewConstructor("MenuBar", "org.objectweb.jac.aspects.gui.web.MenuBar");
         f.setViewConstructor("StatusBar", "org.objectweb.jac.aspects.gui.web.StatusBar");
         f.setViewConstructor("ToolBar", "org.objectweb.jac.aspects.gui.web.ToolBar");
         f.setViewConstructor("Menu", "org.objectweb.jac.aspects.gui.web.Menu");
         
         
         f.setViewConstructor("dateHour", "org.objectweb.jac.aspects.gui.web.DateHourViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("java.util.Date", "org.objectweb.jac.aspects.gui.web.DateViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");

         f.setViewConstructor("java.lang.Float", "org.objectweb.jac.aspects.gui.web.FloatViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("java.lang.Double", "org.objectweb.jac.aspects.gui.web.FloatViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("float", "org.objectweb.jac.aspects.gui.web.FloatViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("double", "org.objectweb.jac.aspects.gui.web.FloatViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         
         f.setViewConstructor("imageURL", "org.objectweb.jac.aspects.gui.web.ImageURLViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("text", "org.objectweb.jac.aspects.gui.web.TextViewer");
         
         // table cells viewers
         f.setViewConstructor("cell:imageURL", "org.objectweb.jac.aspects.gui.web.ImageURLViewer()");
         f.setViewConstructor("cell:java.util.Date", "org.objectweb.jac.aspects.gui.web.DateViewer()");
         f.setViewConstructor("cell:dateHour", "org.objectweb.jac.aspects.gui.web.DateHourViewer()");
         f.setViewConstructor("cell:java.lang.Float", "org.objectweb.jac.aspects.gui.web.FloatViewer()");
         f.setViewConstructor("cell:java.lang.Double", "org.objectweb.jac.aspects.gui.web.FloatViewer()");
         f.setViewConstructor("cell:float", "org.objectweb.jac.aspects.gui.web.FloatViewer()");
         f.setViewConstructor("cell:double", "org.objectweb.jac.aspects.gui.web.FloatViewer()");
         f.setViewConstructor("cell:Reference", "org.objectweb.jac.aspects.gui.web.ReferenceView()");
         f.setViewConstructor("cell:Enum", "org.objectweb.jac.aspects.gui.web.EnumViewer(org.objectweb.jac.util.Enum)");
         f.setViewConstructor("cell:org.objectweb.jac.lib.Attachment", "org.objectweb.jac.aspects.gui.web.AttachmentViewer()");
         f.setViewConstructor("cell:List", "org.objectweb.jac.aspects.gui.web.CompactList()");

         //editors
         f.setViewConstructor("ParameterContainer", "org.objectweb.jac.aspects.gui.web.EditorContainer");
         f.setViewConstructor("PrimitiveTypeEditor", "org.objectweb.jac.aspects.gui.web.PrimitiveFieldEditor");
         f.setViewConstructor("ObjectChooser", "org.objectweb.jac.aspects.gui.web.ObjectChooser");
         f.setViewConstructor("IndexSelector", "org.objectweb.jac.aspects.gui.web.IndexSelector");
         f.setViewConstructor("IndicesSelector", "org.objectweb.jac.aspects.gui.web.IndicesSelector");
         f.setViewConstructor("editor:java.util.Date", "org.objectweb.jac.aspects.gui.web.DateEditor");
         f.setViewConstructor("editor:dateHour", "org.objectweb.jac.aspects.gui.web.DateHourEditor");
         f.setViewConstructor("editor:java.net.URL", "org.objectweb.jac.aspects.gui.web.URLEditor");
         f.setViewConstructor("editor:java.lang.Boolean", "org.objectweb.jac.aspects.gui.web.BooleanEditor");
         f.setViewConstructor("editor:boolean", "org.objectweb.jac.aspects.gui.web.BooleanEditor");
         f.setViewConstructor("editor:text", "org.objectweb.jac.aspects.gui.web.TextEditor");
         f.setViewConstructor("editor:javaCode", "org.objectweb.jac.aspects.gui.web.TextEditor");
         f.setViewConstructor("editor:password", "org.objectweb.jac.aspects.gui.web.PasswordFieldEditor");
         f.setViewConstructor("org.objectweb.jac.lib.Attachment", "org.objectweb.jac.aspects.gui.web.AttachmentViewer(java.lang.Object,java.lang.Object,org.objectweb.jac.core.rtti.FieldItem)");
         f.setViewConstructor("editor:org.objectweb.jac.lib.Attachment", "org.objectweb.jac.aspects.gui.web.AttachmentEditor");
         
         f.setViewConstructor("CollectionItemView", "org.objectweb.jac.aspects.gui.web.CollectionItemView");
      }
   }

   // viewType -> constructor
   // views added in web.acc with setViewConstructor
   Hashtable constructors = new Hashtable();

   /**
    * Create a view of a given type.
    *
    * @param label the name of the view
    * @param viewType the type of the view (for instance, "Object" or "Tree".
    * @param params the parameters to pass to the view constructor
    * @return a view of the demanded type.
    */
   public View createView(String label, String viewType, Object[] params,
                          DisplayContext context) 
      throws UnhandledViewTypeException
   {
      Log.trace("gui.factory",
                "createView(\""+label+"\","+viewType+","+
                Arrays.asList(params)+")");
      AbstractMethodItem constructor = 
         (AbstractMethodItem)constructors.get(viewType);
      if (constructor==null) {
         throw new UnhandledViewTypeException(viewType);
      }
      Log.trace("gui.factory",2,"constructor = "+constructor);
      Object[] parameters;
      Class[] paramTypes = constructor.getParameterTypes();
      if (paramTypes.length>0 && paramTypes[0]==ViewFactory.class) {
         Log.trace("gui.factory",2,"prepending factory to params");
         if (paramTypes.length>1 && paramTypes[1]==DisplayContext.class) {
            Log.trace("gui.factory",2,"prepending context to params");
            parameters = new Object[params.length+2];
            System.arraycopy(params,0,parameters,2,params.length);
            parameters[0] = this;
            parameters[1] = context;
         } else {
            parameters = new Object[params.length+1];
            System.arraycopy(params,0,parameters,1,params.length);
            parameters[0] = this;
         }
      } else {
         parameters = params;
      } 
      Log.trace("gui.factory",2,"invoking constructor..."+Arrays.asList(params));
      if (constructor.getParameterCount()!=parameters.length) {
         throw new RuntimeException("Wrong number of arguments for "+
                                    constructor+" ("+Arrays.asList(parameters)+")");
      }
      View result = (View)constructor.invoke(null,parameters);
      result.setLabel(label);
      result.setFactory(this);
      result.setParameters(params);
      result.setType(viewType);
      result.setContext(context);
      Log.trace("gui.factory",2,"createView("+viewType+","+label+") -> "+result);
      return result;
   }

   /**
    * Convert Strings into the type expected by the view constructor.
    */
   public View createView(String label, String viewType, String[] params,
                          DisplayContext context) 
      throws UnhandledViewTypeException
   {
      Log.trace("gui.factory",
                "createView(\""+label+"\","+viewType+","+
                Arrays.asList(params)+")");
      AbstractMethodItem constructor = 
         (AbstractMethodItem)constructors.get(viewType);
      if (constructor==null) {
         throw new UnhandledViewTypeException(viewType);
      }
      Log.trace("gui.factory","constructor = "+constructor);
      Class[] paramTypes = constructor.getParameterTypes();
      Object[] args = new Object[params.length];
      int offset = 0;
      if (paramTypes.length>0 && paramTypes[0]==ViewFactory.class)
         offset += 1;
      if (paramTypes.length>1 && paramTypes[1]==DisplayContext.class)
         offset += 1;

      for (int i=0; i<params.length;i++) {
         if (String.class.isAssignableFrom(paramTypes[i+offset])) {
            // String: leave as is
            args[i] = params[i];
         } else if (Boolean.class.isAssignableFrom(paramTypes[i+offset]) || 
                    boolean.class.isAssignableFrom(paramTypes[i+offset]) ) {
            args[i] = Boolean.valueOf(params[i]);
         } else if (Object.class.isAssignableFrom(paramTypes[i+offset])) {
            // Object: object name
            args[i] = NameRepository.get().getObject(params[i]);
            Log.trace("gui.factory",params[i]+" -> "+args[i]);
            if (args[i]==null) {
               Log.warning("gui.factory",
                           "Could not find object named "+params[i]);
            }
         }
      }
      return createView(label,viewType,args,context);
   }

   /**
    * Creates a view for an object, using the most specific view
    * constructor available for the type of the object.
    *
    * @param label the label of the view to create
    * @param object the object for which to create a view
    * @param context the display context
    */
   public View createObjectView(String label, Object object,
                                DisplayContext context) {
      if (object==null) {
         return createView(label,"Object",new Object[] {GuiAC.DEFAULT_VIEW,object}, 
                           context);
      }
      ClassItem cl = ClassRepository.get().getClass(object);
      while (cl!=null) {
         if (hasViewerFor(cl.getName())) {
            return createView(label,cl.getName(),
                              new Object[] {object}, context);
         }
         cl = cl.getSuperclass();
      }
      return createView(label,"Object",new Object[] {"default",object}, context);
   }

   /**
    * Creates a view for an object, using the most specific view
    * constructor available for the type of the object.
    *
    * @param label the label of the view to create
    * @param object the object for which to create a view
    * @param substance
    * @param field 
    * @param context the display context
    */
   public View createObjectView(String label, Object object,
                                Object substance, FieldItem field,
                                DisplayContext context) {
      if (object==null) {
         return createView(label,"Object",new Object[] {GuiAC.DEFAULT_VIEW,object}, 
                           context);
      }
      ClassItem cl = ClassRepository.get().getClass(object);
      while (cl!=null) {
         if (hasViewerFor(cl.getName())) {
            return createView(label,cl.getName(),
                              new Object[] {object,substance,field}, context);
         }
         cl = cl.getSuperclass();
      }
      return createView(label,"Object",new Object[] {"default",object}, context);
   }

   public CompositeView createCompositeView(String label, String viewType, 
                                            Object[] params, 
                                            DisplayContext context) 
      throws UnhandledViewTypeException
   {
      return (CompositeView)createView(label,viewType,params,context);
   }
   
   public View createView(String label, String viewType, DisplayContext context) 
      throws UnhandledViewTypeException
   {
      return createView(label,viewType,new Object[0],context);
   }

   public CompositeView createCompositeView(String label, String viewType, 
                                            DisplayContext context) 
      throws UnhandledViewTypeException
   {
      return (CompositeView)createView(label,viewType,context);
   }

   public FieldEditor createEditor(String label, String viewType, 
                                   Object[] params, 
                                   DisplayContext context)
      throws UnhandledViewTypeException
   {
      return (FieldEditor)createView(label,viewType,params,context);
   }
   
   /**
    * Set the constructor for a type of view
    *
    * @param viewType the type of the view
    * @param constructor the constructor to be used to build view of
    * that type. It should take a ViewFactory as the first parameter
    */
   public void setViewConstructor(String viewType, AbstractMethodItem constructor) {
      constructors.put(viewType,constructor);
   }

   public void setViewConstructor(String viewType, String constructor) {
      setViewConstructor(viewType,convert(constructor));
   }


   AbstractMethodItem convert(String str) {
      AbstractMethodItem object=null;
      int index = str.indexOf("(");
      try {
         ClassItem classItem;
         if (index==-1) {
            classItem = ClassRepository.get().getClass(str);
            object = classItem.getConstructor("");
         } else {
            classItem = ClassRepository.get().getClass(
               str.substring(0,index));
            object = classItem.getConstructor(str.substring(index));
         }
      } catch (NoSuchClassException e) {
         if (index!=-1)
            index = str.lastIndexOf(".",index);
         else
            index = str.lastIndexOf(".");
         if (index!=-1) {
            try {
               ClassItem classItem = ClassRepository.get().getClass(
                  str.substring(0,index));
               object = classItem.getAbstractMethod(
                  str.substring(index+1));
            } catch (NoSuchClassException e2) {
               throw new RuntimeException("Failed to convert "+str+
                                          " into a constructor");
            }
         } else {
            throw new RuntimeException("Failed to convert "+str+
                                       " into a constructor");
         }
      }
      return object;
   }

   /**
    * Tells wether a view factory is able to build a view for given type.
    * @param viewType the type of the view
    * @see #setViewConstructor(String,AbstractMethodItem)
    */
   public boolean hasViewerFor(String viewType) {
      return constructors.containsKey(viewType);
   }

   /**
    * Thrown to indicate that a ViewFactory is unable to build a view
    * of the given type because no constructor is associated to this
    * type of view.
    * @see #setViewConstructor(String,AbstractMethodItem)
    */
   public static class UnhandledViewTypeException extends RuntimeException {
      /**
       * @param viewType the type of view that the factory is unable to build.
       */
      public UnhandledViewTypeException(String viewType) {
         super("Unable to build a view of type \""+viewType+"\"");
      }
   }

}
