/*
  Copyright (C) 2001-2003 Renaud Pawlak <renaud@aopsys.com>, 
                          Laurent Martelli <laurent@aopsys.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package org.objectweb.jac.aspects.gui.swing;

import org.objectweb.jac.aspects.gui.FieldEditor;
import org.objectweb.jac.aspects.gui.GuiAC;
import org.objectweb.jac.core.rtti.ClassItem;
import org.objectweb.jac.core.rtti.FieldItem;
import org.objectweb.jac.util.WrappedThrowableException;
import java.awt.Dimension;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JTextField;

/**
 * A Swing editor component for date values.
 */

public class DateEditor extends AbstractFieldEditor
   implements FieldEditor
{
   /** Stores the default format of the date */
   protected SimpleDateFormat format = new SimpleDateFormat(GuiAC.getDateFormat());
   protected JTextField textField;

   /**
    * Constructs a new date editor. */

   public DateEditor(Object substance, FieldItem field) {
      super(substance,field);
      textField = new JTextField(10);
      // swing "bug" workaround : prevent the textField from extending
      // vertically.
      textField.setMaximumSize(new Dimension(1000,20));
      textField.setText(format.format(new Date()));
      textField.addFocusListener(this);
      add(textField);
   }

   public Object getValue() {
      try {
         if (textField.getText().equals(""))
            return null;
         else
            return format.parse(textField.getText());
      } catch (Exception e) {
         throw new WrappedThrowableException(e);
      }
   }

   public void setValue(Object value) {
      super.setValue(value);
      if( value != null )
         textField.setText(format.format((Date)value));
   }

   public void setWidth(int width) {
      super.setWidth(width);
      textField.setColumns(width);
   }

}
