/*
  Copyright (C) 2002-2003 Laurent Martelli <laurent@aopsys.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package org.objectweb.jac.aspects.gui.web;

import org.objectweb.jac.aspects.gui.GuiAC;
import org.objectweb.jac.core.rtti.FieldItem;
import java.io.PrintWriter;
import java.lang.reflect.Constructor;


/**
 * HTML editor for primitive types (int, long, float, double, short) and String
 *
 */
public class PrimitiveFieldEditor  extends AbstractFieldEditor 
   implements HTMLEditor
{
   Class type;
   boolean password;

   public PrimitiveFieldEditor(Object substance, FieldItem field,
                               Class type, boolean password) {
      super(substance,field);
      this.type = type;
      this.password = password;
      width = 15;
   }

   // HTMLEditor interface

   public void genHTML(PrintWriter out) {
      boolean hasDefault = field!=null && GuiAC.hasDefaultValue(field);
      out.print("<INPUT type=\""+(password?"password":"text")+"\""+
                " class=\"editor\""+
                " id=\""+label+"\""+
                " name=\""+label+"\""+
                " style=\"width:"+width+"ex\""+
                /*
                " value=\""+ (value==null || 
                              ((field!=null && field.isPrimitive()) && !hasDefault) 
                              ? "" : value.toString())+"\""*/
                " value=\""+displayedValue()+"\"");
      printAttributes(out);
      out.print(">");
   }

   protected String displayedValue() {
      return value==null ? "" : value.toString();
   }

   protected boolean doReadValue(Object parameter) {
      if (parameter==null)
         return false;
      String string = (String)parameter;
      if ( type == int.class || type == Integer.class ) {
         setValue(new Integer (string));
      } else if ( type == boolean.class || type == Boolean.class) {
         setValue(Boolean.valueOf(string));
      } else if ( type == long.class || type == Long.class ) {
         setValue(new Long (string));
      } else if ( type == float.class || type == Float.class ) {
         setValue(new Float (string));
      } else if ( type == double.class || type == Double.class ) {
         setValue(new Double (string));
      } else if ( type == short.class || type == Short.class ) {
         setValue(new Short (string));
      } else if ( type == byte.class || type == Byte.class ) {
         setValue(new Byte(string));
      } else if ( type == char.class || type == Character.class ) {
         setValue(new Character(string.charAt(0)));
      } else if ( type == String.class ) {
         setValue(string);
      } else {
         try {
            // trying to construct the object from its textual 
            // representation (I think that any class should have
            // a constructor taking a string... this is so helpful...)
            // of course, this will raise an exception most of the time :-(
            Constructor c = type.getConstructor(new Class[] {String.class});
            setValue(c.newInstance(new Object[] {string}));
         } catch( Exception e ) {
            e.printStackTrace();
            throw new RuntimeException("Unhandled type "+type.getName());
         }
      }
      return true;
   }
}

