/*
  Copyright (C) 2002-2003 Laurent Martelli <laurent@aopsys.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package org.objectweb.jac.core;

import org.objectweb.jac.core.rtti.AbstractMethodItem;
import org.objectweb.jac.core.rtti.ClassItem;
import org.objectweb.jac.core.rtti.ClassRepository;
import java.util.Arrays;
import java.util.List;

public class Interaction {
   public Wrappee wrappee;
   public AbstractMethodItem method;
   public Object[] args;
   public int rank;
   public String cur_AC;
   public Object[][] wrappingChain;

   public Interaction(WrappingChain wrappingChain,Wrappee wrappee, 
                      AbstractMethodItem method, Object[] args) {
      this.wrappingChain = wrappingChain==null ? new Object[0][0] : wrappingChain.chain;
      this.wrappee = wrappee;
      this.method = method;
      this.args = args;
      this.rank = 0;
   }

   public Object proceed() {
      rank += 1;
      return Wrapping.nextWrapper(this);
   }

   public Object invoke(Object substance) {
      return method.invoke(substance,args);
   }

   public Class getActualClass() {
      if (wrappee!=null)
         return wrappee.getClass();
      else
         return method.getClassItem().getActualClass();
   }

   ClassItem cli;
   public ClassItem getClassItem() {
      if (cli==null) {
         if (wrappee!=null) {
            cli = ClassRepository.get().getClass(wrappee);
         } else {
            cli = method.getClassItem();
         }
      }
      return cli;
   }

   public String toString() {
      return wrappee+"."+method+"("+Arrays.asList(args)+") rank="+rank;
   }
}
