/*
  Copyright (C) 2002 Laurent Martelli <laurent@aopsys.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package org.objectweb.jac.core;

import java.lang.reflect.Method;


public class WrappingChain {
   /**
    * Each element of this array is a two elements array whose 1st
    * element is the wrapper, and whose 2nd element is the
    * wrappingMethod 
    */
   public Object[][] chain;

   public WrappingChain(Object[][] chain) {
      this.chain = chain;
   }
   public WrappingChain() {
      this.chain = new Object[0][0];
   }
   public void add(int rank,Object[] wrappingMethod) {
      Object[][] newChain = new Object[chain.length+1][2];
      System.arraycopy(chain,0,newChain,0,rank);
      System.arraycopy(chain,rank,newChain,rank+1,chain.length-rank);
      newChain[rank]=wrappingMethod;
      chain = newChain;
   }
   public boolean contains(Object[] wrappingMethod) {
      for (int i=chain.length-1; i>=0; i--) {
         if (chain[i][1]==wrappingMethod[1])
            return true;
      }
      return false;
   }
   public void remove(int rank) {
      Object[][] newChain = new Object[chain.length-1][2];
      System.arraycopy(chain,0,newChain,0,rank);
      System.arraycopy(chain,rank+1,newChain,rank,chain.length-rank-1);
      chain = newChain;      
   }
   public int size() {
      return chain.length;
   }
   public Object[] get(int i) {
      return chain[i];
   }
   public String toString() {
      String result = "[";
      for (int i=0; i<chain.length;i++) {
         if (i!=0) {
            result += ",";
         }
         result += ((Method)chain[i][1]).getName();
      } 
      
      result += "]";
      return result;
   }
}
