/*
  Copyright (C) 2002-2003 Renaud Pawlak <renaud@aopsys.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package org.objectweb.jac.ide;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import org.objectweb.jac.aspects.gui.Actions;
import  java.util.Iterator;
import  java.util.List;
import  java.util.Vector;
import org.objectweb.jac.aspects.gui.Actions;
import org.objectweb.jac.aspects.gui.DisplayContext;
import org.objectweb.jac.core.Wrappee;
import org.objectweb.jac.core.rtti.FieldItem;
import java.io.IOException;

public class Projects {

   public static transient Projects root;
   public static transient TypeRepository types;
   public static transient Plurals plurals;

   public static Boolean notPrimitiveType(Wrappee substance,
                                          FieldItem field,
                                          Object value,
                                          Object[] values)
   {
      Iterator it = Projects.types.getPrimitiveTypes().iterator();
      while(it.hasNext()) {
         Type type = (Type)it.next();
         if (((String)value).equals(type.getName())) {
            return Boolean.FALSE;
         }
      }
      return Boolean.TRUE;
   }

   public static void main(String[] args) {
      root = new Projects();
      types = new TypeRepository();
      plurals = new Plurals();

      // create the primitive types if needed

      if (types.getPrimitiveTypes().size() == 0) {
         types.addPrimitiveType(new Type("void",""));
         types.addPrimitiveType(new Type("boolean",""));
         types.addPrimitiveType(new Type("int",""));
         types.addPrimitiveType(new Type("long",""));
         types.addPrimitiveType(new Type("float",""));
         types.addPrimitiveType(new Type("double",""));

         // create some useful external classes
         types.addExternalClass(new Type("Object","java.lang"));

         Type type;
         type = new Type("String","java.lang");
         types.addExternalClass(type);
         types.addExtendedType(new ExtendedType("text",type));
         types.addExtendedType(new ExtendedType("email",type));
         types.addExtendedType(new ExtendedType("password",type));
         types.addExtendedType(new ExtendedType("javaCode",type));
         types.addExtendedType(new ExtendedType("accCode",type));

         type = new Type("File","java.io");
         types.addExternalClass(type);
         types.addExtendedType(new ExtendedType("directory",type));

         type = new Type("URL","java.net");
         types.addExternalClass(type);
         types.addExtendedType(new ExtendedType("directoryURL",type));
         types.addExtendedType(new ExtendedType("imageURL",type));

         type = new Type("Date","java.util");
         types.addExternalClass(type);
         types.addExtendedType(new ExtendedType("dateHour",type));
      
         type = new Type("Vector","java.util");
         types.addExternalClass(type);

         type = new Type("List","java.util");
         types.addExternalClass(type);

         type = new Type("Map","java.util");
         types.addExternalClass(type);

         type = new Type("HashMap","java.util");
         types.addExternalClass(type);

         type = new Type("Set","java.util");
         types.addExternalClass(type);

         type = new Type("HashSet","java.util");
         types.addExternalClass(type);

      }
   }

   public static void createProject() throws Exception {
      Actions.invoke(root,"addProject");
   }

   public static void staticSetCurrentApplication() {
      Actions.invoke(root,"setCurrentApplication");
   }

   public static void generateCode() throws Exception {
      Iterator it = root.projects.iterator();
      while(it.hasNext()) {
         Project p = (Project)it.next();
         p.generateCode();
      }
   }

   public static Errors compile() throws Exception {
      Iterator it = root.projects.iterator();
      while (it.hasNext()) {
         Project p = (Project)it.next();
         Errors errors = null;
         errors = p.compile();
         if (errors!=null) {
            return errors;
         }
      }
      return null;
   }

   Vector projects = new Vector();
   
   /**
    * Get the value of projects.
    * @return value of projects.
    */
   public List getProjects() {
      return projects;
   }
   
   public void addProject(Project p) {
      projects.add(p);
   }
   
   public void removeProject(Project p) {
      projects.remove(p);
   }

   public String toString() {
      return "projects";
   }

   Application currentApplication = null;
   public void setCurrentApplication(Application application) {
      currentApplication = application;
   }
   public Application getCurrentApplication() {
      return currentApplication;
   }

   /**
    * Starts the current application
    */
   public void startCurrentApplication() throws IOException {
      if (currentApplication!=null) {
         currentApplication.start();
      }
   }

   /**
    * Stops the current application
    */
   public void stopCurrentApplication() {
      if (currentApplication!=null) {
         currentApplication.stop();
      }
   }

   public boolean isNotStarted() {
      return currentApplication!=null && currentApplication.isNotStarted();
   }

   public boolean isStarted() {
      return currentApplication!=null && currentApplication.isStarted();      
   }

}
